__revision__="/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Executor.py 0.96.91.D001 2005/09/08 09:14:36 knight"
import string
from SCons.Debug import logInstanceCreation
import SCons.Util
class Executor:
    if SCons.Memoize.use_memoizer: __metaclass__=SCons.Memoize.Memoized_Metaclass
    def __init__(self, action, env=None, overridelist=[{}], targets=[], sources=[], builder_kw={}):
        if __debug__: logInstanceCreation(self, 'Executor.Executor')
        self.set_action_list(action)
        self.pre_actions = []
        self.post_actions = []
        self.env = env
        self.overridelist = overridelist
        self.targets = targets
        self.sources = sources[:]
        self.builder_kw = builder_kw
    def set_action_list(self, action):
        if not SCons.Util.is_List(action):
            if not action: raise SCons.Errors.UserError, "Executor must have an action."
            action=[action]
        self.action_list=action
    def get_action_list(self):
        return self.pre_actions+self.action_list+self.post_actions
    def get_build_env(self):
        """__cacheable__"""
        overrides = {}
        for odict in self.overridelist: overrides.update(odict)
        import SCons.Defaults
        env = self.env or SCons.Defaults.DefaultEnvironment()
        build_env = env.Override(overrides)
        return build_env
    def get_build_scanner_path(self, scanner):
        env = self.get_build_env()
        try:
            cwd = self.targets[0].cwd
        except (IndexError, AttributeError):
            cwd = None
        return scanner.path(env, cwd, self.targets, self.sources)
    def get_kw(self, kw={}):
        result = self.builder_kw.copy()
        result.update(kw)
        return result
    def do_nothing(self, target, exitstatfunc, kw):
        pass
    def do_execute(self, target, exitstatfunc, kw):
        env = self.get_build_env()
        kw = self.get_kw(kw)
        for act in self.get_action_list(): apply(act, (self.targets, self.sources, env, exitstatfunc), kw)
    def __call__(self, target, exitstatfunc, **kw):
        self.do_execute(target, exitstatfunc, kw)
    def cleanup(self):
        "__reset_cache__"
        pass
    def add_sources(self, sources):
        slist = filter(lambda x, s=self.sources: x not in s, sources)
        self.sources.extend(slist)
    def add_pre_action(self, action):
        self.pre_actions.append(action)
    def add_post_action(self, action):
        self.post_actions.append(action)
    def my_str(self):
        env = self.get_build_env()
        get = lambda action, t=self.targets, s=self.sources, e=env: action.genstring(t, s, e)
        return string.join(map(get, self.get_action_list()), "\n")
    def __str__(self):
        "__cacheable__"
        return self.my_str()
    def nullify(self):
        "__reset_cache__"
        self.do_execute = self.do_nothing
        self.my_str     = lambda S=self: ''
    def get_contents(self):
        """__cacheable__"""
        env = self.get_build_env()
        get = lambda action, t=self.targets, s=self.sources, e=env: action.get_contents(t, s, e)
        return string.join(map(get, self.get_action_list()), "")
    def get_timestamp(self):
        return 0
    def scan_targets(self, scanner):
        self.scan(scanner, self.targets)
    def scan_sources(self, scanner):
        self.scan(scanner, self.sources)
    def scan(self, scanner, node_list):
        env = self.get_build_env()
        select_specific_scanner = lambda t: (t[0], t[1].select(t[0]))
        remove_null_scanners = lambda t: not t[1] is None
        add_scanner_path = lambda t, s=self: (t[0], t[1], s.get_build_scanner_path(t[1]))
        if scanner:
            scanner_list = map(lambda src, s=scanner: (src, s), node_list)
        else:
            kw = self.get_kw()
            get_initial_scanners = lambda src, e=env, kw=kw: (src, src.get_scanner(e, kw))
            scanner_list = map(get_initial_scanners, node_list)
            scanner_list = filter(remove_null_scanners, scanner_list)

        scanner_list = map(select_specific_scanner, scanner_list)
        scanner_list = filter(remove_null_scanners, scanner_list)
        scanner_path_list = map(add_scanner_path, scanner_list)
        deps = []
        for node, scanner, path in scanner_path_list:
            deps.extend(node.get_implicit_deps(env, scanner, path))

        for tgt in self.targets:
            tgt.add_to_implicit(deps)

    def get_missing_sources(self):
        """__cacheable__"""
        return filter(lambda s: s.missing(), self.sources)

    def get_unignored_sources(self, ignore):
        """__cacheable__"""
        sourcelist = self.sources
        if ignore:
            sourcelist = filter(lambda s, i=ignore: not s in i, sourcelist)
        return sourcelist

    def process_sources(self, func, ignore=[]):
        """__cacheable__"""
        return map(func, self.get_unignored_sources(ignore))
_Executor = Executor
class Null(_Executor):
    def __init__(self, *args, **kw):
        if __debug__: logInstanceCreation(self, 'Executor.Null')
        kw['action'] = []
        apply(_Executor.__init__, (self,), kw)
    def get_build_env(self):
        class NullEnvironment:
            def get_scanner(self, key):
                return None
        return NullEnvironment()
    def get_build_scanner_path(self):
        return None
    def cleanup(self):
        pass
if SCons.Memoize.use_old_memoization():
    _Base = Executor
    class Executor(SCons.Memoize.Memoizer, _Base):
        def __init__(self, *args, **kw):
            SCons.Memoize.Memoizer.__init__(self)
            apply(_Base.__init__, (self,)+args, kw)
