__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Node/__init__.py 0.96.91.D001 2005/09/08 09:14:36 knight"
import copy
import string
import UserList
from SCons.Debug import logInstanceCreation
import SCons.Executor
import SCons.SConsign
import SCons.Util
no_state = 0
pending = 1
executing = 2
up_to_date = 3
executed = 4
failed = 5
stack = 6
StateString = {
    0 : "0",
    1 : "pending",
    2 : "executing",
    3 : "up_to_date",
    4 : "executed",
    5 : "failed",
    6 : "stack",
}
implicit_cache = 0
implicit_deps_unchanged = 0
implicit_deps_changed = 0
def do_nothing(node): pass
Annotate = do_nothing
class NodeInfo:
    def __init__(self):
        pass
    def __cmp__(self, other):
        return cmp(self.__dict__, other.__dict__)
    def update(self, node):
        pass
    def merge(self, other):
        for key, val in other.__dict__.items():
            self.__dict__[key] = val
class BuildInfo:
    def __init__(self, node):
        self.ninfo = node.new_ninfo()
        self.bsourcesigs = []
        self.bdependsigs = []
        self.bimplicitsigs = []
        self.bactsig = None
    def __cmp__(self, other):
        return cmp(self.ninfo, other.ninfo)
    def merge(self, other):
        for key, val in other.__dict__.items():
            try:
                merge = self.__dict__[key].merge
            except (AttributeError, KeyError):
                self.__dict__[key] = val
            else:
                merge(val)
class Node:
    if SCons.Memoize.use_memoizer: __metaclass__ = SCons.Memoize.Memoized_Metaclass
    class Attrs:
        pass
    def __init__(self):
        if __debug__: logInstanceCreation(self, 'Node.Node')
        self.sources = []
        self.sources_dict = {}
        self.depends = []
        self.depends_dict = {}
        self.ignore = []
        self.ignore_dict = {}
        self.implicit = None
        self.waiting_parents = []
        self.wkids = None

        self.env = None
        self.state = no_state
        self.precious = None
        self.always_build = None
        self.found_includes = {}
        self.includes = None
        self.attributes = self.Attrs()
        self.side_effect = 0
        self.side_effects = []
        self.pre_actions = []
        self.post_actions = []
        self.linked = 0
        Annotate(self)

    def get_suffix(self):
        return ''

    def get_build_env(self):
        """__cacheable__"""
        return self.get_executor().get_build_env()

    def get_build_scanner_path(self, scanner):
        return self.get_executor().get_build_scanner_path(scanner)

    def set_executor(self, executor):
        self.executor = executor

    def get_executor(self, create=1):
        try:
            executor = self.executor
        except AttributeError:
            if not create:
                raise
            try:
                act = self.builder.action
            except AttributeError:
                executor = SCons.Executor.Null(targets=[self])
            else:
                executor = SCons.Executor.Executor(act, self.env or self.builder.env, [self.builder.overrides], [self], self.sources)
            self.executor = executor
        return executor

    def executor_cleanup(self):
        try:
            executor = self.get_executor(create=None)
        except AttributeError:
            pass
        else:
            executor.cleanup()

    def reset_executor(self):
        try:
            delattr(self, 'executor')
        except AttributeError:
            pass

    def retrieve_from_cache(self):
        return 0
        
    def build(self, **kw):
        def exitstatfunc(stat, node=self):
            if stat:
                msg = "Error %d" % stat
                raise SCons.Errors.BuildError(node=node, errstr=msg)
        executor = self.get_executor()
        apply(executor, (self, exitstatfunc), kw)

    def built(self):
        for parent in self.waiting_parents:
            parent.implicit = None
            parent.del_binfo()
        
        try:
            new = self.binfo
        except AttributeError:
            new = None

        self.clear()
        
        if new:
            if hasattr(new.ninfo, 'csig'):
                self.get_csig()
            else:
                new.ninfo.update(self)
                self.binfo = new
            self.store_info(self.binfo)

    def add_to_waiting_parents(self, node):
        self.waiting_parents.append(node)

    def call_for_all_waiting_parents(self, func):
        func(self)
        for parent in self.waiting_parents:
            parent.call_for_all_waiting_parents(func)

    def postprocess(self):
        self.executor_cleanup()

    def clear(self):
        """__reset_cache__"""
        self.executor_cleanup()
        self.del_binfo()
        try:
            delattr(self, '_calculated_sig')
        except AttributeError:
            pass
        self.includes = None
        self.found_includes = {}
        self.implicit = None

        self.waiting_parents = []

    def visited(self):
        pass

    def builder_set(self, builder):
        "__cache_reset__"
        self.builder = builder

    def has_builder(self):
        try:
            b = self.builder
        except AttributeError:
            self.builder = None
            b = self.builder
        return not b is None

    def set_explicit(self, is_explicit):
        self.is_explicit = is_explicit

    def has_explicit_builder(self):
        try:
            return self.is_explicit
        except AttributeError:
            self.is_explicit = None
            return self.is_explicit

    def get_builder(self, default_builder=None):
        try:
            return self.builder
        except AttributeError:
            return default_builder

    multiple_side_effect_has_builder = has_builder

    def is_derived(self):
        """__cacheable__"""
        return self.has_builder() or self.side_effect

    def is_pseudo_derived(self):
        return 0

    def alter_targets(self):
        return [], None

    def get_found_includes(self, env, scanner, path):
        return []

    def get_implicit_deps(self, env, scanner, path):
        if not scanner: return []
        scanner = scanner.select(self)

        nodes = [self]
        seen = {}
        seen[self] = 1
        deps = []
        while nodes:
            n = nodes.pop(0)
            d = filter(lambda x, seen=seen: not seen.has_key(x),
                       n.get_found_includes(env, scanner, path))
            if d:
                deps.extend(d)
                for n in d:
                    seen[n] = 1
                nodes.extend(scanner.recurse_nodes(d))

        return deps

    def get_scanner(self, env, kw={}):
        return env.get_scanner(self.scanner_key())

    def get_source_scanner(self, node):
        scanner = None
        try:
            scanner = self.builder.source_scanner
        except AttributeError:
            pass
        if not scanner:
            scanner = self.get_scanner(self.get_build_env())
        if scanner:
            scanner = scanner.select(node)
        return scanner

    def add_to_implicit(self, deps):
        if not hasattr(self, 'implicit') or self.implicit is None:
            self.implicit = []
            self.implicit_dict = {}
            self._children_reset()
        self._add_child(self.implicit, self.implicit_dict, deps)

    def scan(self):
        if not self.implicit is None: return
        self.implicit = []
        self.implicit_dict = {}
        self._children_reset()
        if not self.has_builder(): return
        build_env = self.get_build_env()

        if implicit_cache and not implicit_deps_changed:
            implicit = self.get_stored_implicit()
            if implicit:
                factory = build_env.get_factory(self.builder.source_factory)
                implicit = map(factory, implicit)
                self._add_child(self.implicit, self.implicit_dict, implicit)
                calc = build_env.get_calculator()
                if implicit_deps_unchanged or self.current(calc):
                    return
                self.implicit = []
                self.implicit_dict = {}
                self._children_reset()
                self.del_binfo()

        executor = self.get_executor()
        executor.scan_sources(self.builder.source_scanner)
        scanner = self.builder.target_scanner
        if scanner:
            executor.scan_targets(scanner)

    def scanner_key(self):
        return None

    def env_set(self, env, safe=0):
        if safe and self.env:
            return
        self.env = env

    def calculator(self):
        import SCons.Defaults
        env = self.env or SCons.Defaults.DefaultEnvironment()
        return env.get_calculator()

    def calc_signature(self, calc=None):
        """__cacheable__"""
        if self.is_derived():
            import SCons.Defaults

            env = self.env or SCons.Defaults.DefaultEnvironment()
            if env.use_build_signature():
                return self.get_bsig(calc)
        elif not self.rexists():
            return None
        return self.get_csig(calc)

    def new_ninfo(self):
        return NodeInfo()

    def new_binfo(self):
        return BuildInfo(self)

    def get_binfo(self):
        try:
            return self.binfo
        except AttributeError:
            self.binfo = self.new_binfo()
            return self.binfo

    def del_binfo(self):
        try:
            delattr(self, 'binfo')
        except AttributeError:
            pass

    def gen_binfo(self, calc=None, scan=1):
        """__cacheable__"""

        if calc is None: calc = self.calculator()

        binfo = self.get_binfo()

        if scan: self.scan()

        executor = self.get_executor()
        def calc_signature(node, calc=calc):
            return node.calc_signature(calc)

        sources = executor.process_sources(None, self.ignore)
        sourcesigs = executor.process_sources(calc_signature, self.ignore)

        depends = self.depends
        implicit = self.implicit or []

        if self.ignore:
            depends = filter(self.do_not_ignore, depends)
            implicit = filter(self.do_not_ignore, implicit)

        dependsigs = map(calc_signature, depends)
        implicitsigs = map(calc_signature, implicit)

        sigs = sourcesigs + dependsigs + implicitsigs

        if self.has_builder():
            binfo.bact = str(executor)
            binfo.bactsig = calc.module.signature(executor)
            sigs.append(binfo.bactsig)

        binfo.bsources = sources
        binfo.bdepends = depends
        binfo.bimplicit = implicit

        binfo.bsourcesigs = sourcesigs
        binfo.bdependsigs = dependsigs
        binfo.bimplicitsigs = implicitsigs

        binfo.ninfo.bsig = calc.module.collect(filter(None, sigs))

        return binfo

    def get_bsig(self, calc=None):
        binfo = self.get_binfo()
        try:
            return binfo.ninfo.bsig
        except AttributeError:
            self.binfo = self.gen_binfo(calc)
            return self.binfo.ninfo.bsig

    def get_csig(self, calc=None):
        binfo = self.get_binfo()
        try:
            return binfo.ninfo.csig
        except AttributeError:
            if calc is None:
                calc = self.calculator()
            csig = binfo.ninfo.csig = calc.module.signature(self)
            return csig
    def store_info(self, obj):
        pass
    def get_stored_info(self):
        return None
    def get_stored_implicit(self):
        return None
    def set_precious(self, precious = 1):
        self.precious = precious
    def set_always_build(self, always_build = 1):
        self.always_build = always_build
    def exists(self):
        return 1
    def rexists(self):
        return self.exists()
    def missing(self):
        """__cacheable__"""
        return not self.is_derived() and \
               not self.is_pseudo_derived() and \
               not self.linked and \
               not self.rexists()
    
    def prepare(self):
        l = self.depends
        if not self.implicit is None:
            l = l + self.implicit
        missing_sources = self.get_executor().get_missing_sources() \
                          + filter(lambda c: c.missing(), l)
        if missing_sources:
            desc = "Source `%s' not found, needed by target `%s'." % (missing_sources[0], self)
            raise SCons.Errors.StopError, desc
    def remove(self):
        return None
    def add_dependency(self, depend):
        try:
            self._add_child(self.depends, self.depends_dict, depend)
        except TypeError, e:
            e = e.args[0]
            if SCons.Util.is_List(e):
                s = map(str, e)
            else:
                s = str(e)
            raise SCons.Errors.UserError("attempted to add a non-Node dependency to %s:\n\t%s is a %s, not a Node" % (str(self), s, type(e)))

    def add_ignore(self, depend):
        try:
            self._add_child(self.ignore, self.ignore_dict, depend)
        except TypeError, e:
            e = e.args[0]
            if SCons.Util.is_List(e):
                s = map(str, e)
            else:
                s = str(e)
            raise SCons.Errors.UserError("attempted to ignore a non-Node dependency of %s:\n\t%s is a %s, not a Node" % (str(self), s, type(e)))
    def add_source(self, source):
        try:
            self._add_child(self.sources, self.sources_dict, source)
        except TypeError, e:
            e = e.args[0]
            if SCons.Util.is_List(e):
                s = map(str, e)
            else:
                s = str(e)
            raise SCons.Errors.UserError("attempted to add a non-Node as source of %s:\n\t%s is a %s, not a Node" % (str(self), s, type(e)))

    def _add_child(self, collection, dict, child):
        if type(child) is not type([]):
            child = [child]
        for c in child:
            if not isinstance(c, Node):
                raise TypeError, c
        added = None
        for c in child:
            if not dict.has_key(c):
                collection.append(c)
                dict[c] = 1
                added = 1
        if added:
            self._children_reset()

    def add_wkid(self, wkid):
        if self.wkids != None:
            self.wkids.append(wkid)

    def _children_reset(self):
        "__cache_reset__"
        self.executor_cleanup()

    def do_not_ignore(self, node):
        return node not in self.ignore

    def _all_children_get(self):
        if self.implicit is None:
            return self.sources + self.depends
        else:
            return self.sources + self.depends + self.implicit

    def _children_get(self):
        "__cacheable__"
        children = self._all_children_get()
        if self.ignore:
            children = filter(self.do_not_ignore, children)
        return children

    def all_children(self, scan=1):
        if scan:
            self.scan()
        return self._all_children_get()

    def children(self, scan=1):
        if scan:
            self.scan()
        return self._children_get()

    def set_state(self, state):
        self.state = state

    def get_state(self):
        return self.state

    def current(self, calc=None):
        return None

    def children_are_up_to_date(self, calc=None):
        self.binfo = self.gen_binfo(calc)
        if self.always_build:
            return None
        state = 0
        for kid in self.children(None):
            s = kid.get_state()
            if s and (not state or s > state):
                state = s
        return (state == 0 or state == SCons.Node.up_to_date)

    def is_literal(self):
        return 1

    def render_include_tree(self):
        if self.is_derived() and self.env:
            env = self.get_build_env()
            for s in self.sources:
                scanner = self.get_source_scanner(s)
                path = self.get_build_scanner_path(scanner)
                def f(node, env=env, scanner=scanner, path=path):
                    return node.get_found_includes(env, scanner, path)
                return SCons.Util.render_tree(s, f, 1)
        else:
            return None

    def get_abspath(self):
        return str(self)

    def for_signature(self):
        return str(self)

    def get_string(self, for_signature):
        if for_signature:
            return self.for_signature()
        return str(self)

    def get_subst_proxy(self):
        return self

    def explain(self):
        if not self.exists():
            return "building `%s' because it doesn't exist\n" % self

        old = self.get_stored_info()
        if old is None:
            return None

        def dictify(result, kids, sigs):
            for k, s in zip(kids, sigs):
                result[k] = s

        try:
            osig = {}
            dictify(osig, old.bsources, old.bsourcesigs)
            dictify(osig, old.bdepends, old.bdependsigs)
            dictify(osig, old.bimplicit, old.bimplicitsigs)
        except AttributeError:
            return "Cannot explain why `%s' is being rebuilt: No previous build information found\n" % self

        new = self.get_binfo()

        nsig = {}
        dictify(nsig, new.bsources, new.bsourcesigs)
        dictify(nsig, new.bdepends, new.bdependsigs)
        dictify(nsig, new.bimplicit, new.bimplicitsigs)
        old_bkids = old.bsources + old.bdepends + old.bimplicit
        new_bkids = new.bsources + new.bdepends + new.bimplicit
        stringify = lambda s, E=self.dir.Entry: str(E(s))
        lines = []
        removed = filter(lambda x, nk=new_bkids: not x in nk, old_bkids)
        if removed:
            removed = map(stringify, removed)
            fmt = "`%s' is no longer a dependency\n"
            lines.extend(map(lambda s, fmt=fmt: fmt % s, removed))

        for k in new_bkids:
            if not k in old_bkids:
                lines.append("`%s' is a new dependency\n" % stringify(k))
            elif osig[k] != nsig[k]:
                lines.append("`%s' changed\n" % stringify(k))

        if len(lines) == 0 and old_bkids != new_bkids:
            lines.append("the dependency order changed:\n" +
                         "%sold: %s\n" % (' '*15, map(stringify, old_bkids)) +
                         "%snew: %s\n" % (' '*15, map(stringify, new_bkids)))

        if len(lines) == 0:
            def fmt_with_title(title, strlines):
                lines = string.split(strlines, '\n')
                sep = '\n' + ' '*(15 + len(title))
                return ' '*15 + title + string.join(lines, sep) + '\n'
            if old.bactsig != new.bactsig:
                if old.bact == new.bact:
                    lines.append("the contents of the build action changed\n" +
                                 fmt_with_title('action: ', new.bact))
                else:
                    lines.append("the build action changed:\n" +
                                 fmt_with_title('old: ', old.bact) +
                                 fmt_with_title('new: ', new.bact))

        if len(lines) == 0: return "rebuilding `%s' for unknown reasons\n" % self

        preamble = "rebuilding `%s' because" % self
        if len(lines) == 1:
            return "%s %s"  % (preamble, lines[0])
        else:
            lines = ["%s:\n" % preamble] + lines
            return string.join(lines, ' '*11)
l = [1]
ul = UserList.UserList([2])
try:
    l.extend(ul)
except TypeError:
    def NodeList(l):
        return l
else:
    class NodeList(UserList.UserList):
        def __str__(self):
            return str(map(str, self.data))
del l
del ul

if SCons.Memoize.use_old_memoization():
    _Base = Node
    class Node(SCons.Memoize.Memoizer, _Base):
        def __init__(self, *args, **kw):
            apply(_Base.__init__, (self,)+args, kw)
            SCons.Memoize.Memoizer.__init__(self)
def get_children(node, parent): return node.children()
def ignore_cycle(node, stack): pass
def do_nothing(node, parent): pass

class Walker:
    def __init__(self, node, kids_func=get_children,
                             cycle_func=ignore_cycle,
                             eval_func=do_nothing):
        self.kids_func = kids_func
        self.cycle_func = cycle_func
        self.eval_func = eval_func
        node.wkids = copy.copy(kids_func(node, None))
        self.stack = [node]
        self.history = {}
        self.history[node] = None

    def next(self):
        while self.stack:
            if self.stack[-1].wkids:
                node = self.stack[-1].wkids.pop(0)
                if not self.stack[-1].wkids:
                    self.stack[-1].wkids = None
                if self.history.has_key(node):
                    self.cycle_func(node, self.stack)
                else:
                    node.wkids = copy.copy(self.kids_func(node, self.stack[-1]))
                    self.stack.append(node)
                    self.history[node] = None
            else:
                node = self.stack.pop()
                del self.history[node]
                if node:
                    if self.stack:
                        parent = self.stack[-1]
                    else:
                        parent = None
                    self.eval_func(node, parent)
                return node
        return None
    def is_done(self):
        return not self.stack
arg2nodes_lookups = []
