__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/SConf.py 0.96.91.D001 2005/09/08 09:14:36 knight"
import os
import re
import string
import StringIO
import sys
import traceback
import types
import SCons.Action
import SCons.Builder
import SCons.Errors
import SCons.Node.FS
import SCons.Taskmaster
import SCons.Util
import SCons.Warnings
import SCons.Conftest
SCons.Conftest.LogInputFiles = 0
SCons.Conftest.LogErrorMessages = 0
dryrun = 0
AUTO=0
FORCE=1
CACHE=2
cache_mode = AUTO
def SetCacheMode(mode):
    global cache_mode
    if mode == "auto": cache_mode = AUTO
    elif mode == "force": cache_mode = FORCE
    elif mode == "cache": cache_mode = CACHE
    else: raise ValueError, "SCons.SConf.SetCacheMode: Unknown mode " + mode
progress_display = SCons.Util.display
def SetProgressDisplay(display):
    global progress_display
    progress_display = display
SConfFS = None
_ac_build_counter = 0
_ac_config_logs = {}
_ac_config_hs   = {}
sconf_global = None
def _createConfigH(target, source, env):
    t = open(str(target[0]), "w")
    defname = re.sub('[^A-Za-z0-9_]', '_', string.upper(str(target[0])))
    t.write("""#ifndef %(DEFNAME)s_SEEN
#define %(DEFNAME)s_SEEN

""" % {'DEFNAME' : defname})
    t.write(source[0].get_contents())
    t.write("""
#endif /* %(DEFNAME)s_SEEN */
""" % {'DEFNAME' : defname})
    t.close()

def _stringConfigH(target, source, env):
    return "scons: Configure: creating " + str(target[0])

def CreateConfigHBuilder(env):
    if len(_ac_config_hs) == 0:
        return
    action = SCons.Action.Action(_createConfigH,
                                 _stringConfigH)
    sconfigHBld = SCons.Builder.Builder(action=action)
    env.Append( BUILDERS={'SConfigHBuilder':sconfigHBld} )
    for k in _ac_config_hs.keys():
        env.SConfigHBuilder(k, env.Value(_ac_config_hs[k]))
    
class SConfWarning(SCons.Warnings.Warning):
    pass
SCons.Warnings.enableWarningClass(SConfWarning)

class SConfError(SCons.Errors.UserError):
    def __init__(self,msg):
        SCons.Errors.UserError.__init__(self,msg)

class ConfigureDryRunError(SConfError):
    def __init__(self,target):
        if not isinstance(target, SCons.Node.FS.File): msg = 'Cannot create configure directory "%s" within a dry-run.' % str(target)
        else: msg = 'Cannot update configure test "%s" within a dry-run.' % str(target)
        SConfError.__init__(self,msg)
class ConfigureCacheError(SConfError):
    def __init__(self,target):
        SConfError.__init__(self, '"%s" is not yet built and cache is forced.' % str(target))
def _createSource( target, source, env ):
    fd = open(str(target[0]), "w")
    fd.write(source[0].get_contents())
    fd.close()
def _stringSource( target, source, env ):
    import string
    return (str(target[0]) + ' <-\n  |' + string.replace( source[0].get_contents(), '\n', "\n  |" ) )

BooleanTypes = [types.IntType]
if hasattr(types, 'BooleanType'): BooleanTypes.append(types.BooleanType)
class SConfBuildInfo(SCons.Node.FS.BuildInfo):
    result = None
    string = None
    def __init__(self, node, result, string, sig):
        SCons.Node.FS.BuildInfo.__init__(self, node)
        self.result = result
        self.string = string
        self.ninfo.bsig = sig
class Streamer:
    def __init__(self, orig):
        self.orig = orig
        self.s = StringIO.StringIO()
    def write(self, str):
        if self.orig:
            self.orig.write(str)
        self.s.write(str)
    def writelines(self, lines):
        for l in lines:
            self.write(l + '\n')
    def getvalue(self):
        return self.s.getvalue()

class SConfBuildTask(SCons.Taskmaster.Task):
    def display(self, message):
        if sconf_global.logstream:
            sconf_global.logstream.write("scons: Configure: " + message + "\n")
    def display_cached_string(self, bi):
        if not isinstance(bi, SConfBuildInfo): SCons.Warnings.warn(SConfWarning, "The stored build information has an unexpected class.")
        else:
            self.display("The original builder output was:\n" + string.replace("  |" + str(bi.string), "\n", "\n  |"))
    def failed(self):
        exc_type = self.exc_info()[0]
        if issubclass(exc_type, SConfError): raise
        elif issubclass(exc_type, SCons.Errors.BuildError): pass
        else:
            self.display('Caught exception while building "%s":\n' %
                         self.targets[0])
            try:
                excepthook = sys.excepthook
            except AttributeError:
                def excepthook(type, value, tb):
                    import traceback
                    traceback.print_tb(tb)
                    print type, value
            apply(excepthook, self.exc_info())
        return SCons.Taskmaster.Task.failed(self)

    def collect_node_states(self):
        is_up_to_date = 1
        cached_error = 0
        cachable = 1
        for t in self.targets:
            bi = t.get_stored_info()
            c_bi = isinstance(bi, SConfBuildInfo)
            if c_bi:
                if cache_mode == CACHE:
                    t.set_state(SCons.Node.up_to_date)
                else:
                    new_bsig = t.calc_signature(sconf_global.calc)
                    if t.env.use_build_signature():
                        old_bsig = bi.ninfo.bsig
                    else:
                        old_bsig = bi.ninfo.csig
                    is_up_to_date = (is_up_to_date and new_bsig == old_bsig)
                cached_error = cached_error or bi.result
            else:
                cachable = 0
                is_up_to_date = 0
        return (is_up_to_date, cached_error, cachable)

    def execute(self):
        sconf = sconf_global

        is_up_to_date, cached_error, cachable = self.collect_node_states()

        if cache_mode == CACHE and not cachable:
            raise ConfigureCacheError(self.targets[0])
        elif cache_mode == FORCE:
            is_up_to_date = 0

        if cached_error and is_up_to_date:
            self.display("Building \"%s\" failed in a previous run and all "
                         "its sources are up to date." % str(self.targets[0]))
            self.display_cached_string(self.targets[0].get_stored_info())
            raise SCons.Errors.BuildError
        elif is_up_to_date:            
            self.display("\"%s\" is up to date." % str(self.targets[0]))
            self.display_cached_string(self.targets[0].get_stored_info())
        elif dryrun:
            raise ConfigureDryRunError(self.targets[0])
        else:
            s = sys.stdout = sys.stderr = Streamer(sys.stdout)
            try:
                env = self.targets[0].get_build_env()
                env['PSTDOUT'] = env['PSTDERR'] = s
                try:
                    sconf.cached = 0
                    self.targets[0].build()
                finally:
                    sys.stdout = sys.stderr = env['PSTDOUT'] = \
                                 env['PSTDERR'] = sconf.logstream
            except KeyboardInterrupt:
                raise
            except SystemExit:
                exc_value = sys.exc_info()[1]
                raise SCons.Errors.ExplicitExit(self.targets[0],exc_value.code)
            except:
                for t in self.targets:
                    sig = t.calc_signature(sconf.calc)
                    string = s.getvalue()
                    binfo = SConfBuildInfo(t,1,string,sig)
                    t.dir.sconsign().set_entry(t.name, binfo)
                raise
            else:
                for t in self.targets:
                    sig = t.calc_signature(sconf.calc)
                    string = s.getvalue()
                    binfo = SConfBuildInfo(t,0,string,sig)
                    t.dir.sconsign().set_entry(t.name, binfo)

class SConf:
    def __init__(self, env, custom_tests = {}, conf_dir='#/.sconf_temp', log_file='#/config.log', config_h = None, _depth = 0): 
        global SConfFS
        if not SConfFS:
            SConfFS = SCons.Node.FS.default_fs or SCons.Node.FS.FS(env.fs.pathTop)
        if not sconf_global is None: raise (SCons.Errors.UserError, "Only one SConf object may be active at one time")
        self.env = env
        if log_file != None: self.logfile = SConfFS.File(log_file)
        else: self.logfile = None
        self.logstream = None
        self.lastTarget = None
        self.depth = _depth
        self.cached = 0

        default_tests = {
                 'CheckFunc'          : CheckFunc,
                 'CheckType'          : CheckType,
                 'CheckHeader'        : CheckHeader,
                 'CheckCHeader'       : CheckCHeader,
                 'CheckCXXHeader'     : CheckCXXHeader,
                 'CheckLib'           : CheckLib,
                 'CheckLibWithHeader' : CheckLibWithHeader
               }
        self.AddTests(default_tests)
        self.AddTests(custom_tests)
        self.confdir = SConfFS.Dir(conf_dir)
        self.calc = None
        if not config_h is None: config_h = SConfFS.File(config_h)
        self.config_h = config_h
        self._startup()

    def Finish(self):
        self._shutdown()
        return self.env

    def BuildNodes(self, nodes):
        if self.logstream != None:
            oldStdout = sys.stdout
            sys.stdout = self.logstream
            oldStderr = sys.stderr
            sys.stderr = self.logstream
        old_fs_dir = SConfFS.getcwd()
        old_os_dir = os.getcwd()
        SConfFS.chdir(SConfFS.Top, change_os_dir=1)
        ret = 1
        try:
            save_max_drift = SConfFS.get_max_drift()
            SConfFS.set_max_drift(0)
            tm = SCons.Taskmaster.Taskmaster(nodes, SConfBuildTask)
            jobs = SCons.Job.Jobs(1, tm )
            jobs.run()
            for n in nodes:
                state = n.get_state()
                if (state != SCons.Node.executed and
                    state != SCons.Node.up_to_date):
                    ret = 0
        finally:
            SConfFS.set_max_drift(save_max_drift)
            os.chdir(old_os_dir)
            SConfFS.chdir(old_fs_dir, change_os_dir=0)
            if self.logstream != None:
                sys.stdout = oldStdout
                sys.stderr = oldStderr
        return ret
    def pspawn_wrapper(self, sh, escape, cmd, args, env):
        return self.pspawn(sh, escape, cmd, args, env, self.logstream, self.logstream)

    def TryBuild(self, builder, text = None, extension = ""):
        global _ac_build_counter
        try:
            self.pspawn = self.env['PSPAWN']
        except KeyError:
            raise SCons.Errors.UserError('Missing PSPAWN construction variable.')
        try:
            save_spawn = self.env['SPAWN']
        except KeyError:
            raise SCons.Errors.UserError('Missing SPAWN construction variable.')
        nodesToBeBuilt = []
        f = "conftest_" + str(_ac_build_counter)
        pref = self.env.subst( builder.builder.prefix )
        suff = self.env.subst( builder.builder.suffix )
        target = self.confdir.File(pref + f + suff)
        try:
            self.env['SPAWN'] = self.pspawn_wrapper
            sourcetext = self.env.Value(text)

            if text != None:
                textFile = self.confdir.File(f + extension)
                textFileNode = self.env.SConfSourceBuilder(target=textFile, source=sourcetext)
                nodesToBeBuilt.extend(textFileNode)
                source = textFileNode
            else: source = None

            nodes = builder(target = target, source = source)
            if not SCons.Util.is_List(nodes):
                nodes = [nodes]
            nodesToBeBuilt.extend(nodes)
            result = self.BuildNodes(nodesToBeBuilt)

        finally:
            self.env['SPAWN'] = save_spawn

        _ac_build_counter = _ac_build_counter + 1
        if result: self.lastTarget = nodes[0]
        else: self.lastTarget = None

        return result

    def TryAction(self, action, text = None, extension = ""):
        builder = SCons.Builder.Builder(action=action)
        self.env.Append( BUILDERS = {'SConfActionBuilder' : builder} )
        ok = self.TryBuild(self.env.SConfActionBuilder, text, extension)
        del self.env['BUILDERS']['SConfActionBuilder']
        if ok:
            outputStr = self.lastTarget.get_contents()
            return (1, outputStr)
        return (0, "")

    def TryCompile( self, text, extension):
        return self.TryBuild(self.env.Object, text, extension)

    def TryLink( self, text, extension ):
        return self.TryBuild(self.env.Program, text, extension )

    def TryRun(self, text, extension ):
        ok = self.TryLink(text, extension)
        if( ok ):
            prog = self.lastTarget
            pname = str(prog)
            output = SConfFS.File(pname+'.out')
            node = self.env.Command(output, prog, [ [ pname, ">", "${TARGET}"] ])
            ok = self.BuildNodes(node)
            if ok:
                outputStr = output.get_contents()
                return( 1, outputStr)
        return (0, "")

    class TestWrapper:
        def __init__(self, test, sconf):
            self.test = test
            self.sconf = sconf
        def __call__(self, *args, **kw):
            if not self.sconf.active:
                raise (SCons.Errors.UserError, "Test called after sconf.Finish()")
            context = CheckContext(self.sconf)
            ret = apply(self.test, (context,) +  args, kw)
            if not self.sconf.config_h is None:
                self.sconf.config_h_text = self.sconf.config_h_text + context.config_h
            context.Result("error: no result")
            return ret

    def AddTest(self, test_name, test_instance):
        setattr(self, test_name, SConf.TestWrapper(test_instance, self))

    def AddTests(self, tests):
        for name in tests.keys():
            self.AddTest(name, tests[name])

    def _createDir( self, node ):
        dirName = str(node)
        if dryrun:
            if not os.path.isdir( dirName ):
                raise ConfigureDryRunError(dirName)
        else:
            if not os.path.isdir( dirName ):
                os.makedirs( dirName )
                node._exists = 1

    def _startup(self):
        global _ac_config_logs
        global sconf_global
        global SConfFS
        
        self.lastEnvFs = self.env.fs
        self.env.fs = SConfFS
        self._createDir(self.confdir)
        self.confdir.up().add_ignore( [self.confdir] )

        if self.logfile != None and not dryrun:
            if _ac_config_logs.has_key(self.logfile):
                log_mode = "a"
            else:
                _ac_config_logs[self.logfile] = None
                log_mode = "w"
            self.logstream = open(str(self.logfile), log_mode)
            self.logfile.dir.add_ignore( [self.logfile] )

            tb = traceback.extract_stack()[-3-self.depth]
            old_fs_dir = SConfFS.getcwd()
            SConfFS.chdir(SConfFS.Top, change_os_dir=0)
            self.logstream.write('file %s,line %d:\n\tConfigure(confdir = %s)\n' %
                                 (tb[0], tb[1], str(self.confdir)) )
            SConfFS.chdir(old_fs_dir)
        else: 
            self.logstream = None
        action = SCons.Action.Action(_createSource, _stringSource)
        sconfSrcBld = SCons.Builder.Builder(action=action)
        self.env.Append( BUILDERS={'SConfSourceBuilder':sconfSrcBld} )
        self.config_h_text = _ac_config_hs.get(self.config_h, "")
        self.active = 1
        sconf_global = self

    def _shutdown(self):
        global sconf_global, _ac_config_hs

        if not self.active:
            raise SCons.Errors.UserError, "Finish may be called only once!"
        if self.logstream != None and not dryrun:
            self.logstream.write("\n")
            self.logstream.close()
            self.logstream = None
        blds = self.env['BUILDERS']
        del blds['SConfSourceBuilder']
        self.env.Replace( BUILDERS=blds )
        self.active = 0
        sconf_global = None
        if not self.config_h is None:
            _ac_config_hs[self.config_h] = self.config_h_text
        self.env.fs = self.lastEnvFs

class CheckContext:
    def __init__(self, sconf):
        self.sconf = sconf
        self.did_show_result = 0

        self.vardict = {}
        self.havedict = {}
        self.headerfilename = None
        self.config_h = ""
    def Message(self, text):
        self.Display(text)
        self.sconf.cached = 1
        self.did_show_result = 0
    def Result(self, res):
        if type(res) in BooleanTypes:
            if res: text = "yes"
            else: text = "no"
        elif type(res) == types.StringType:
            text = res
        else:
            raise TypeError, "Expected string, int or bool, got " + str(type(res))

        if self.did_show_result == 0:
            self.Display(text + "\n")
            self.did_show_result = 1

    def TryBuild(self, *args, **kw):
        return apply(self.sconf.TryBuild, args, kw)

    def TryAction(self, *args, **kw):
        return apply(self.sconf.TryAction, args, kw)

    def TryCompile(self, *args, **kw):
        return apply(self.sconf.TryCompile, args, kw)

    def TryLink(self, *args, **kw):
        return apply(self.sconf.TryLink, args, kw)

    def TryRun(self, *args, **kw):
        return apply(self.sconf.TryRun, args, kw)

    def __getattr__( self, attr ):
        if( attr == 'env' ):
            return self.sconf.env
        elif( attr == 'lastTarget' ):
            return self.sconf.lastTarget
        else:
            raise AttributeError, "CheckContext instance has no attribute '%s'" % attr

    def BuildProg(self, text, ext):
        self.sconf.cached = 1
        return not self.TryBuild(self.env.Program, text, ext)

    def CompileProg(self, text, ext):
        self.sconf.cached = 1
        return not self.TryBuild(self.env.Object, text, ext)

    def AppendLIBS(self, lib_name_list):
        oldLIBS = self.env.get( 'LIBS', [] )
        self.env.Append(LIBS = lib_name_list)
        return oldLIBS

    def SetLIBS(self, val):
        oldLIBS = self.env.get( 'LIBS', [] )
        self.env.Replace(LIBS = val)
        return oldLIBS

    def Display(self, msg):
        if self.sconf.cached:
            msg = "(cached) " + msg
            self.sconf.cached = 0
        progress_display(msg, append_newline=0)
        self.Log("scons: Configure: " + msg + "\n")

    def Log(self, msg):
        if self.sconf.logstream != None:
            self.sconf.logstream.write(msg)

def CheckFunc(context, function_name, header = None, language = None):
    res = SCons.Conftest.CheckFunc(context, function_name, header = header, language = language)
    context.did_show_result = 1
    return not res

def CheckType(context, type_name, includes = "", language = None):
    res = SCons.Conftest.CheckType(context, type_name,
                                        header = includes, language = language)
    context.did_show_result = 1
    return not res

def createIncludesFromHeaders(headers, leaveLast, include_quotes = '""'):
    if not SCons.Util.is_List(headers):
        headers = [headers]
    l = []
    if leaveLast:
        lastHeader = headers[-1]
        headers = headers[:-1]
    else:
        lastHeader = None
    for s in headers:
        l.append("#include %s%s%s\n"
                 % (include_quotes[0], s, include_quotes[1]))
    return string.join(l, ''), lastHeader

def CheckHeader(context, header, include_quotes = '<>', language = None):
    prog_prefix, hdr_to_check = createIncludesFromHeaders(header, 1, include_quotes)
    res = SCons.Conftest.CheckHeader(context, hdr_to_check, prog_prefix, language = language, include_quotes = include_quotes)
    context.did_show_result = 1
    return not res

def CheckCHeader(context, header, include_quotes = '""'):
    return CheckHeader(context, header, include_quotes, language = "C")
def CheckCXXHeader(context, header, include_quotes = '""'):
    return CheckHeader(context, header, include_quotes, language = "C++")
def CheckLib(context, library = None, symbol = "main", autoadd = 1, header = None, language = None):
    if library == []: library = [None]
    if not SCons.Util.is_List(library): library = [library]
    res = SCons.Conftest.CheckLib(context, library, symbol, header = header, language = language, autoadd = autoadd)
    context.did_show_result = 1
    return not res
def CheckLibWithHeader(context, libs, header, language, call = "main();", autoadd = 1):
    prog_prefix, dummy = createIncludesFromHeaders(header, 0)
    if libs == []: libs = [None]
    if not SCons.Util.is_List(libs): libs = [libs]
    res = SCons.Conftest.CheckLib(context, libs, "main", prog_prefix, call = call, language = language, autoadd = autoadd)
    context.did_show_result = 1
    return not res
