__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Scanner/__init__.py 0.96.91.D001 2005/09/08 09:14:36 knight"

import re
import string

import SCons.Node.FS
import SCons.Sig
import SCons.Util
class _Null:
    pass
_null = _Null
def Scanner(function, *args, **kw):
    if SCons.Util.is_Dict(function): return apply(Selector, (function,) + args, kw)
    else: return apply(Base, (function,) + args, kw)
class _Binder:
    def __init__(self, bindval):
        self._val = bindval
    def __call__(self):
        return self._val
    def __str__(self):
        return str(self._val)
BinderDict = {}
def Binder(path):
    try:
        return BinderDict[path]
    except KeyError:
        b = _Binder(path)
        BinderDict[path] = b
        return b
class FindPathDirs:
    def __init__(self, variable):
        self.variable = variable
    def __call__(self, env, dir, target=None, source=None, argument=None):
        try:
            path = env[self.variable]
        except KeyError:
            return ()
        path = env.subst_path(path, target=target, source=source)
        path_tuple = tuple(env.fs.Rfindalldirs(path, dir))
        return Binder(path_tuple)

class Base:
    if SCons.Memoize.use_memoizer:
        __metaclass__ = SCons.Memoize.Memoized_Metaclass
    def __init__(self,
                 function,
                 name = "NONE",
                 argument = _null,
                 skeys = [],
                 path_function = None,
                 node_class = SCons.Node.FS.Entry,
                 node_factory = None,
                 scan_check = None,
                 recursive = None):
        self.function = function
        self.path_function = path_function
        self.name = name
        self.argument = argument
        self.skeys = skeys
        self.node_class = node_class
        self.node_factory = node_factory
        self.scan_check = scan_check
        if callable(recursive):
            self.recurse_nodes = recursive
        elif recursive:
            self.recurse_nodes = self._recurse_all_nodes
        else:
            self.recurse_nodes = self._recurse_no_nodes

    def path(self, env, dir=None, target=None, source=None):
        if not self.path_function:
            return ()
        if not self.argument is _null:
            return self.path_function(env, dir, target, source, self.argument)
        else:
            return self.path_function(env, dir, target, source)

    def __call__(self, node, env, path = ()):
        if self.scan_check and not self.scan_check(node, env):
            return []

        if not self.argument is _null:
            list = self.function(node, env, path, self.argument)
        else:
            list = self.function(node, env, path)
        kw = {}
        if hasattr(node, 'dir'):
            kw['directory'] = node.dir
        node_factory = env.get_factory(self.node_factory)
        nodes = []
        for l in list:
            if self.node_class and not isinstance(l, self.node_class):
                l = apply(node_factory, (l,), kw)
            nodes.append(l)
        return nodes

    def __cmp__(self, other):
        try:
            return cmp(self.__dict__, other.__dict__)
        except AttributeError:
            return cmp(self.__dict__, other)

    def __hash__(self):
        return id(self)

    def __str__(self):
        return self.name

    def add_skey(self, skey):
        """Add a skey to the list of skeys"""
        self.skeys.append(skey)

    def get_skeys(self, env=None):
        if SCons.Util.is_String(self.skeys):
            return env.subst_list(self.skeys)[0]
        return self.skeys

    def select(self, node):
        return self

    def _recurse_all_nodes(self, nodes):
        return nodes

    def _recurse_no_nodes(self, nodes):
        return []

    recurse_nodes = _recurse_no_nodes

if SCons.Memoize.use_old_memoization():
    _Base = Base
    class Base(SCons.Memoize.Memoizer, _Base):
        "Cache-backed version of Scanner Base"
        def __init__(self, *args, **kw):
            apply(_Base.__init__, (self,)+args, kw)
            SCons.Memoize.Memoizer.__init__(self)


class Selector(Base):
    def __init__(self, dict, *args, **kw):
        apply(Base.__init__, (self, None,)+args, kw)
        self.dict = dict

    def __call__(self, node, env, path = ()):
        return self.select(node)(node, env, path)

    def select(self, node):
        try:
            return self.dict[node.scanner_key()]
        except KeyError:
            return None
    def add_scanner(self, skey, scanner):
        self.dict[skey] = scanner

class Current(Base):
    def __init__(self, *args, **kw):
        def current_check(node, env):
            c = not node.has_builder() or node.current(env.get_calculator())
            return c
        kw['scan_check'] = current_check
        apply(Base.__init__, (self,) + args, kw)

class Classic(Current):
    def __init__(self, name, suffixes, path_variable, regex, *args, **kw):

        self.cre = re.compile(regex, re.M)

        def _scan(node, env, path=(), self=self):
            node = node.rfile()
            if not node.exists():
                return []
            return self.scan(node, path)

        kw['function'] = _scan
        kw['path_function'] = FindPathDirs(path_variable)
        kw['recursive'] = 1
        kw['skeys'] = suffixes
        kw['name'] = name

        apply(Current.__init__, (self,) + args, kw)

    def find_include(self, include, source_dir, path):
        "__cacheable__"
        if callable(path): path = path()
        n = SCons.Node.FS.find_file(include, (source_dir,) + tuple(path))
        return n, include

    def sort_key(self, include):
        return SCons.Node.FS._my_normcase(include)

    def scan(self, node, path=()):
        "__cacheable__"
        if node.includes != None:
            includes = node.includes
        else:
            includes = self.cre.findall(node.get_contents())
            node.includes = includes

        nodes = []
        source_dir = node.get_dir()
        for include in includes:
            n, i = self.find_include(include, source_dir, path)

            if n is None:
                SCons.Warnings.warn(SCons.Warnings.DependencyWarning,
                                    "No dependency generated for file: %s (included from: %s) -- file not found" % (i, node))
            else:
                sortkey = self.sort_key(include)
                nodes.append((sortkey, n))

        nodes.sort()
        nodes = map(lambda pair: pair[1], nodes)
        return nodes

class ClassicCPP(Classic):
    def find_include(self, include, source_dir, path):
        "__cacheable__"
        if callable(path): path = path()
        if include[0] == '"': paths = Binder( (source_dir,) + tuple(path) )
        else: paths = Binder( tuple(path) + (source_dir,) )
        n = SCons.Node.FS.find_file(include[1], paths)
        return n, include[1]
    def sort_key(self, include):
        return SCons.Node.FS._my_normcase(string.join(include))
